"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableGlobalProxyAgent = exports.bindHttpMethod = void 0;
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const semver_1 = __importDefault(require("semver"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('global-proxy-agent');
function bindHttpMethod(originalMethod, agent, forceGlobalAgent) {
    return (...args) => {
        let url;
        let options;
        let callback;
        if (typeof args[0] === 'string' || args[0] instanceof URL) {
            url = args[0];
            if (typeof args[1] === 'function') {
                options = {};
                callback = args[1];
            }
            else {
                options = {
                    ...args[1],
                };
                callback = args[2];
            }
        }
        else {
            options = {
                ...args[0],
            };
            callback = args[1];
        }
        if (forceGlobalAgent) {
            options.agent = agent;
        }
        else {
            if (!options.agent) {
                options.agent = agent;
            }
            if (options.agent === http_1.default.globalAgent ||
                options.agent === https_1.default.globalAgent) {
                options.agent = agent;
            }
        }
        if (url) {
            return originalMethod(url, options, callback);
        }
        else {
            return originalMethod(options, callback);
        }
    };
}
exports.bindHttpMethod = bindHttpMethod;
const httpGet = http_1.default.get;
const httpRequest = http_1.default.request;
const httpsGet = https_1.default.get;
const httpsRequest = https_1.default.request;
const httpGlobalAgent = http_1.default.globalAgent;
const httpsGlobalAgent = https_1.default.globalAgent;
/**
 * enableGlobalProxyAgent replaces http.globalAgent/https.globalAgent with provided proxyAgent
 * @param proxyAgent
 * @param options
 * @see https://github.com/gajus/global-agent
 * @return disableGlobalProxyAgent - Disables globalProxyAgent
 */
function enableGlobalProxyAgent(proxyAgent, options) {
    const forceGlobalAgent = options?.forceGlobalAgent ?? true;
    const skipHttp = options?.http === false;
    const skipHttps = options?.https === false;
    // Overriding globalAgent was added in v11.7.
    // @see https://nodejs.org/uk/blog/release/v11.7.0/
    if (semver_1.default.gte(process.version, 'v11.7.0')) {
        if (!skipHttp) {
            debug('replace http.globalAgent');
            // @see https://github.com/facebook/flow/issues/7670
            http_1.default.globalAgent = proxyAgent;
        }
        if (!skipHttps) {
            debug('replace https.globalAgent');
            // @ts-expect-error Node.js version compatibility
            https_1.default.globalAgent = proxyAgent;
        }
    }
    // The reason this logic is used in addition to overriding http(s).globalAgent
    // is because there is no guarantee that we set http(s).globalAgent variable
    // before an instance of http(s).Agent has been already constructed by someone,
    // e.g. Stripe SDK creates instances of http(s).Agent at the top-level.
    // @see https://github.com/gajus/global-agent/pull/13
    //
    // We still want to override http(s).globalAgent when possible to enable logic
    // in `bindHttpMethod`.
    if (semver_1.default.gte(process.version, 'v10.0.0')) {
        if (!skipHttp) {
            debug('bindHttpMethod', 'http.get', forceGlobalAgent);
            // @ts-expect-error seems like we are using wrong type for httpAgent
            http_1.default.get = bindHttpMethod(httpGet, proxyAgent, forceGlobalAgent);
            debug('bindHttpMethod', 'http.request', forceGlobalAgent);
            // @ts-expect-error seems like we are using wrong type for httpAgent
            http_1.default.request = bindHttpMethod(
            // @ts-expect-error seems like we are using wrong type for httpAgent
            httpRequest, proxyAgent, forceGlobalAgent);
        }
        if (!skipHttps) {
            debug('bindHttpMethod', 'https.get', forceGlobalAgent);
            // @ts-expect-error seems like we are using wrong type for httpsAgent
            https_1.default.get = bindHttpMethod(httpsGet, proxyAgent, forceGlobalAgent);
            debug('bindHttpMethod', 'https.request', forceGlobalAgent);
            // @ts-expect-error seems like we are using wrong type for httpsAgent
            https_1.default.request = bindHttpMethod(
            // @ts-expect-error seems like we are using wrong type for httpsAgent
            httpsRequest, proxyAgent, forceGlobalAgent);
        }
    }
    else {
        // eslint-disable-next-line no-console
        console.warn('attempt to initialize global proxy-agent in unsupported Node.js version was ignored');
    }
    return () => {
        if (semver_1.default.gte(process.version, 'v11.7.0')) {
            if (!skipHttp) {
                debug('restore http.globalAgent');
                http_1.default.globalAgent = httpGlobalAgent;
            }
            if (!skipHttps) {
                debug('restore https.globalAgent');
                https_1.default.globalAgent = httpsGlobalAgent;
            }
        }
        if (semver_1.default.gte(process.version, 'v10.0.0')) {
            if (!skipHttp) {
                debug('restore http.get', forceGlobalAgent);
                http_1.default.get = httpGet;
                debug('restore http.request', forceGlobalAgent);
                http_1.default.request = httpRequest;
            }
            if (!skipHttps) {
                debug('restore https.get', forceGlobalAgent);
                https_1.default.get = httpsGet;
                debug('restore https.request', forceGlobalAgent);
                https_1.default.request = httpsRequest;
            }
        }
    };
}
exports.enableGlobalProxyAgent = enableGlobalProxyAgent;
//# sourceMappingURL=index.js.map